-- windows doesn't handle non-latin alphabets correctly (well, windows sucks in general)
-- so we first create alphanumeric-only anonymizer, then try to replace the basic version
-- with cyrylic-aware one, which fails on Windows reducing anonymization effectivness
create or replace function source_alphabet() returns text as
$$
select 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'::text;
$$ language sql;
create or replace function source_alphabet() returns text as
$$
select 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyząęśćźńżĄĘŚĆŹŃŻ0АаБбВвГгДдЕеЁёЖжЗзИиЙйКкЛлМмНнОоПпРрСсТтУуФфХхЦцЧчШшЩщЪъЫыЬьЭэЮюЯя'::text;
$$ language sql;

create or replace function random_letters() returns text as
$$
declare
    chars  text[];
    result text    := '';
    i      integer := 0;
    length integer;
BEGIN
    select regexp_split_to_array(source_alphabet(), '') into chars;
    select array_length(chars, 1) into length;
    for i in 1..length
        loop
            result := result || chars[1 + random() * (length - 1)];
        end loop;
    return result;
END;
$$ language plpgsql;

create or replace function random_digits() returns text as
$$
declare
    chars_input text    := '0123456789';
    chars       text[];
    result      text    := '';
    i           integer := 0;
    length      integer;
BEGIN
    select regexp_split_to_array(chars_input, '') into chars;
    select array_length(chars, 1) into length;
    for i in 1..length
        loop
            result := result || chars[1 + random() * (length - 1)];
        END loop;
    return result;
END;
$$ language plpgsql;

create or replace function anon_text(s text) RETURNS text AS
$$
select case when length(s) > 10 then translate(s, source_alphabet() || '0123456789', random_letters() || random_digits()) else translate(s, source_alphabet(), random_letters()) || random()::text end;
$$ LANGUAGE sql;

create or replace function anon_email(s text) RETURNS text AS
$$
select md5(random()::text) || substring(s from position('@' in s)) || '.disabled';
$$ LANGUAGE sql;

create or replace function check_proper_db() returns text as
$$
BEGIN
    if (current_database() != 'anonymize')
    then
        raise exception 'You must use a copy of production database into temporary database "anonymize"';
    end if;
    return 'Ok, this is correct database';
END;
$$ language plpgsql;

create or replace function anon_all_tables() returns integer as
$$
DECLARE
    x integer;
    r information_schema.columns%rowtype;
BEGIN
    x := 0;
    -- table_schema, table_name, column_name
    FOR r IN
        SELECT * FROM information_schema.columns where column_name like '%email%' and data_type in ('character varying', 'text') and is_updatable = 'YES' and table_schema != 'pg_catalog' and table_name != 'system_configuration'
        LOOP
            x := x + 1;
            EXECUTE 'update ' || quote_ident(r.table_schema) || '.' || quote_ident(r.table_name) || ' SET ' || quote_ident(r.column_name) || ' = anon_email(' || quote_ident(r.column_name) || ');';
        END LOOP;

    FOR r IN
        SELECT * FROM information_schema.columns where (column_name like '%notes%' or column_name like '%password%' or column_name like 'account_owner_%' or column_name like 'address_address%' or column_name like '%\_phone%' or column_name like '%name\_normalized%' or column_name like '%sales\_%' or column_name in ('last_name', 'first_name', 'full_name', 'customer_name', 'enrolment_directory')) and data_type in ('character varying', 'text') and is_updatable = 'YES' and table_schema != 'pg_catalog' and table_name != 'system_configuration'
        LOOP
            x := x + 1;
            EXECUTE 'update ' || quote_ident(r.table_schema) || '.' || quote_ident(r.table_name) || ' SET ' || quote_ident(r.column_name) || ' = anon_text(' || quote_ident(r.column_name) || ');';
        END LOOP;
    return x;
END;
$$ language plpgsql;
